#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#define RET_OK	0
#define RET_WA	5
#define	RET_PE	4
#define RET_CE	6

int N;
char S1[10001], S2[10001];
int P1,P2;

int Pos;
int Ops;

char* U[10001]; 

FILE *in, *out;

void dispose()
{
	if(in) fclose(in);
	if(out) fclose(out);

	for(int i = 0; i < 10000; i++)
		if(U[i])
		{
			delete [] U[i];
			U[i] = 0;
		}
}

int execed = 0;


void exec(char op)
{
	execed = 1;
	if(op == '<')
	{
		Pos--;
		Ops++;
	}
	else if(op == '>')
	{
		Pos++;
		Ops++;
	}
	else if(op == 'v')
	{
		if(S1[Pos] == '-')
		{
			fprintf(stderr,"Attempt to 'v' on '-'.\n");
			dispose();
			exit(RET_WA);
		}
		S1[Pos] = '-';
		Ops++;
	}
	else if(op == 'x')
	{
		if(S1[Pos] == '.')
		{
			fprintf(stderr,"Attempt to 'x' on '.'.\n");
			dispose();
			exit(RET_WA);
		}
		S1[Pos] = '.';
		Ops++;
	}

	if(Pos < 0 || Pos >= N)
	{
		fprintf(stderr,"Pos != [1..Length]\n");
		dispose();
		exit(RET_WA);
	}
	else if(Ops > 1000000)
	{
		fprintf(stderr,"OpCount > 1E6.\n");
		dispose();
		exit(RET_WA);
	}
}

char tempmacro[6001];
int tempindex;

int main(int n, char ** argv)
{
	
	if(n < 3)
	{
		fprintf(stderr, "Need 2 arguments.\nt [input] [output]\n");
		return RET_CE;
	}

	in = fopen(/*"input.txt"*/argv[1],"r");
	if(!in)
	{
		fprintf(stderr,"Can't open input file.\n");
		return RET_CE;
	}
	out = fopen(argv[2]/*"output.txt"*/,"r");
	if(!out)
	{
		fprintf(stderr,"Can't open output file.\n");
		return RET_CE;
	}

	fscanf(in,"%d\n%s\n%d\n%s\n%d",&N,S1,&P1,S2,&P2);
	P1--, P2--;

	Pos = P1;

	char s[11];
	
	int lines = 0;

	while(!feof(out))
	{
		fgets(s,10,out);

		lines++;

		if(s[0] == '<' || s[0] == '>' || s[0] == 'v' || s[0] == 'x')
			exec(s[0]);
		else if(s[0] == 's')
			break;
		else if(s[0] == 'M')
		{
			if(execed)
			{
				fprintf(stderr,"Macros can be defined only before code.\n");
				dispose();
				return RET_WA;
			}
			int m = atoi(s+1);
			if(m < 1 || m >= 10000)
			{
				fprintf(stderr,"Macro name '%d' not possible.\n",m);
				dispose();
				return RET_WA;
			}
			if(U[m])
			{
				fprintf(stderr,"Macro '%d' redefinition.\n",m);
				dispose();
				return RET_WA;
			}
			tempindex = 0;
			while(1)
			{
				if(feof(out))
				{
					fprintf(stderr,"EOF in macro '%d'.\n",m);
					dispose();
					return RET_WA;
				}
				fgets(s,10,out);
		
				lines++;

				if(s[0] == 'E')
					break;
				else if(s[0] == '<' || s[0] == '>' || s[0] == 'v' || s[0] == 'x')
				{
					tempmacro[tempindex++] = s[0];
					if(tempindex >= 6000)
					{
						fprintf(stderr,"Too long macro '%d'.\n",m);
						dispose();
						return RET_WA;
					}
				}
				else
				{
					fprintf(stderr,"Unknown command in macro '%d'.\n",m);
					dispose();
					return RET_WA;
				}
			}
			tempmacro[tempindex] = 0;
			U[m] = new char[tempindex+1];
			strcpy(U[m],tempmacro);
		}
		else if(s[0] == '#')
		{
			int m = atoi(s+1);
			if(m < 1 || m >= 10000)
			{
				fprintf(stderr,"Macro name '%d' not possible.\n",m);
				dispose();
				return RET_WA;
			}
			if(!U[m])
			{
				fprintf(stderr,"Macro '%d' not defined.\n",m);
				dispose();
				return RET_WA;
			}
			char* s = U[m];
			while(*s)
				exec(*s++);
		}
		else
		{
			fprintf(stderr,"Garbage in output file.\n");
			dispose();
			return RET_WA;
		}

		if(lines > 500+(N/2))
		{
			fprintf(stderr,"Too long program.\n");
			dispose();
			exit(RET_WA);
		}
	}

	dispose();

	for(int i = 0; i < N; i++)
		if(S1[i] != S2[i])
		{
			fprintf(stderr,"Result does not match required.\n");
			return RET_WA;
		}

	//printf("Accepted, %d operations, %d lines.\n",Ops,lines);

	return RET_OK;
}
